//Computes marginal income distributions for parents and children, and absolute upward mobility
//Author: Jimmy Narang, Maximilian Hell

//Uncomment the line below for debugging
//capture program drop absmob
program define absmob_min, rclass
	
	*************************************
	***		   INITIALIZATION		  ***
	*************************************
	version 14.1
	
	syntax  anything(id="compute" name = cmd) ///
	, cohorts(numlist asc integer >=1940 <= 1984) ///
	[parent(string) kid(string) income(string) copula(string) DEBug(string) dir(string) ADJust_for_zeroes]
	
	local par_opts = "dummystr, `parent'"
	local kid_opts = "dummystr, `kid'"
	local inc_opts = "dummystr, `income'"
	local dir_opts = "dummystr, `dir'"
	
	parse_cmd_options `cmd'
	parse_parent_options `par_opts'
	parse_kid_options `kid_opts'
	parse_income_options `inc_opts'
	parse_dir_options `dir_opts'
	
	//store the returned s-values in locals
	if inlist(s(compute), "par_marginal", "par", "all") local compute_parents = "true" 
	if inlist(s(compute), "kid_marginal", "kid", "all") local compute_kids = "true" 
	if inlist(s(compute), "mobility", "mob", "all") local compute_absmob = "true" 
	
	local income_age_min = s(income_age_min)
	local income_age_max = s(income_age_max)
	local parent_age_min = s(parent_age_min)
	local parent_age_max = s(parent_age_max)
		
	local representative_parent = s(rep_parent)
	local par_gender = s(par_gender)
	
	local kid_gender = s(kid_gender)
	local kid_age = s(kid_age)
	local immigrants = s(immigrants)
	
	local inc = s(measure)
	local inflator = s(inflator)
	local baseyear = s(baseyear)
	local divby = s(divby)
	local taxby = s(taxby)
	local raiseby = s(raiseby)
	
	local debug = cond("`debug'" == "off", "f", "true")	
	
	local adjust_for_zeroes = cond("`adjust_for_zeroes'" == "adjust_for_zeroes", "true", "f")	
	
	if "`copula'" == "" local copula = "copula_base_adjusted"
		
	if "`debug'" == "true"{
		capture log close
		set trace on
		set tracedepth 1
		set more off
		log using "$absmob_outdir/absmob_log", replace
		sreturn list
	}
	
	* Years for which Census and CPS data are available
	local census_start = 1940
	local census_end   = 2010
	local cps_start    = 1970
	local cps_end      = 2014

	//even though census_start is 1950, we can obtain incomes for 1941-1949 from 1950
	local census_start_special = 1941 

	//validate if we have data to generate results for the cohorts specified
	foreach c in `cohorts' {
		
		if "`compute_parents'" == "true"{
			if !(`census_start' <= `c' & `c' <= `census_end'){
				di as error "cohorts() invalid -- par_marginal cannot be generated for cohort `c'"
				exit 197
			}
		}	
		
		if "`compute_kids'" == "true"{
			if !( `cps_start' <= (`c' + `kid_age') & (`c' + `kid_age') <= `cps_end'){
				di as error "cohorts() invalid -- kid_marginal cannot be generated for cohort `c' and kid age `kid_age'"
				exit 197
			}	
		}
	}
		
	* As convention, we NEVER cd out of this directory. 
	* Instead, use full paths to read/write outside it.
	cd "$absmob_workingdir"

	local start_decade = 10*floor(`census_start' / 10)
	local end_decade = 10*floor(`census_end' / 10)


	*************************************
	***				PARENTS			  ***
	*************************************
			
	if "`compute_parents'" == "true" {		
		
		* generate representative parent ids for each decade
		forval census_year = `start_decade' (10) `end_decade' {
			
			use "$absmob_indir/census`census_year'_families.dta", clear 
			generate_rep_id "`representative_parent'" "`par_gender'" "`census_year'"
			tempfile census`census_year'_families
			save `census`census_year'_families'
			
			use "$absmob_indir/census`census_year'_couples.dta", clear 
			generate_rep_id "`representative_parent'" "`par_gender'" "`census_year'"
			tempfile census`census_year'_couples
			save `census`census_year'_couples'
			
			save, replace
		}
		
		* Now create samples for each cohort
		foreach cohort in `cohorts' {

			//The "focal" census for a cohort is the census where kids born in a cohort first show up.
			//For example, focal census for 1961, 1969 or 1970 is 1970
			local focal_census = 10*ceil(`cohort' / 10 )
			
			//For each year in which the census was conducted (i.e., for each decade), check whether we need
			//to extract any data from that census to construct the parent sample for this particular cohort.
								
			forval census_year = `start_decade' (10) `end_decade' {
								
				//NOTE: Nothing needs to happen w.r.t par_gender -- 
				//rep. parent is already set to father or mother in the loop at start of this phase, 
				//and all invalid rows have been dropped accordingly
				
				local delta = `census_year' - `cohort'
				
				if `delta' >= 0 {					
					//This is a "Future" census w.r.t the Focal census
					
					use `census`census_year'_families', clear 
					
					//TODO: do we need these? The flags won't work if we generate par_marginal separately
					if "`kid_gender'" == "male" keep if sex_child == 1
					if "`kid_gender'" == "female" keep if sex_child == 2
					
					//Parents extracted from a "Future" census should adhere to 3 constraints:
					//First, their child should've been born in year = cohort
					keep if age_child == `delta'
					
					//Second, their age (at the time of the census) should lie between income_age_min and income_age_max,
					//with the below exception:					
					if "`census_year'" == "`start_decade'" {
						//Since there's no earlier census to pull records from, we also extract age ranges that 
						//we'd have pulled from a previous census
						local income_age_max_earliest = `parent_age_max' - (`cohort' - `census_year')											
						keep if rep_age >= `income_age_min' & rep_age <= `income_age_max_earliest'
					}
					else {						
						keep if rep_age >= `income_age_min' & rep_age <= `income_age_max'
					}
				
					//Third, their age at the time of the birth of their child (i.e., in year = cohort)
					//should lie between parent_age_min and parent_age_max
					gen par_age_at_childbirth = rep_age - age_child
					keep if par_age_at_childbirth  >= `parent_age_min' & par_age_at_childbirth <= `parent_age_max'
					drop par_age_at_childbirth
				}
				
				else {													
					//This is a "Past" census w.r.t the Focal census
					
					//Note that we CAN'T apply gender restrictions by child, unlike "future"
					//TODO: how to handle this bias?
					
					//We don't really know which individuals from the past will go on to have a child by year = 'cohort'.
					//So we can't apply the 1st constraint above. Instead, we assume ALL parents who meet age criteria 
					//go on to have kids.
					
					use `census`census_year'_couples', clear 
									
					//Same as check #2 in the "future" case
					if "`census_year'" == "`start_decade'" {
						//Since there's no earlier census to pull records from, we also extract age ranges that 
						//we'd have pulled from a previous census if it existed
						local income_age_max_earliest = `parent_age_max' - (`cohort' - `census_year')
						keep if rep_age >= `income_age_min' & rep_age <= `income_age_max_earliest'
					}
					else {						
						keep if rep_age >= `income_age_min' & rep_age <= `income_age_max'
					}										
										
					//Same as check #3 in the "future" case (because age_child == delta)
					gen par_age_at_childbirth = rep_age - `delta'
					keep if par_age_at_childbirth  >= `parent_age_min' & par_age_at_childbirth <= `parent_age_max'
					drop par_age_at_childbirth
				}
				
				//These Macros (n_from_focal) will be used to calculate substitution weights later (see note below)
				su rep_age
				local n_from`census_year' = _N
					
				***************
				preserve
				
				use `census`focal_census'_families', clear
				
				local child_age = `focal_census' - `cohort'
				local delta = `focal_census' - `census_year'
				
				//These are the same 3 constraints as above				
				count if age_child == `child_age' & ///
						rep_age >= (`income_age_min' + `delta') & rep_age <= (`income_age_max' + `delta') & ///
						rep_age - `child_age' >= `parent_age_min' & rep_age - `child_age' <= `parent_age_max'
				
				local n_from_focal = r(N)
				
				restore
				***************
				//To construct the parent sample, we need to reweight parents from an older or a newer census 
				//to match the number of parents they represent (substitute) in the focal ("current") census.
				//See the Data section in the paper for an explanation.
				
				gen weights = `n_from_focal' / `n_from`census_year''
				replace weights = 1 if `census_year' == `focal_census'
				replace weights = round(weights*1000000) //weights are meant to be integers
				
				keep year id marst id_spouse rep_id age rep_age rep_sex sex weights famsize age_spouse sex_spouse ///
					nchild incwage incwage_spouse inctot inctot_spouse ftotinc nonwage incwelfr incwelfr_spouse imputed imputed_spouse
				
				tempfile sample_from`census_year'
				save `sample_from`census_year''	
				
				if (`n_from`census_year'' == 0){
					//No rows extracted from this census; pointless to print
					continue
				}				
			} //end foreach census_year
			
			//combine the extracts from each census to create the parent-sample for this cohort
			clear
			forval census_year = `start_decade' (10) `end_decade' {				
				append using `sample_from`census_year''
			}
			tempfile sample_`cohort'
			save `sample_`cohort'', replace

		} //end foreach cohort
		
		*******************END PHASE ************************

		foreach cohort in `cohorts'{

			use `sample_`cohort'', clear

			//Generate the "income" variable
			generate_income "`inc'" "census" "`census_year'" "`par_gender'" "`divby'" "`raiseby'"			
			
			//Scale income by inflator
			inflate_income "`inflator'" "$absmob_indir" "`baseyear'"
			
			//compute income post-tax, if necessary
			if "`taxby'" == "mtr"{
				
				//We need to scale down income back to the cohort-year, 
				//because compute_tax expects incomes in terms of the year for which tax is being computed
				local cohort_inf = `cohort' - 1
				inflate_income_const "$absmob_indir/`inflator'" "income" "`baseyear'" "`cohort_inf'"
				
				compute_tax `cohort'
				replace income = income - ftaxmtr
				
				//Now, scale incomes back				
				inflate_income_const "$absmob_indir/`inflator'" "income" "`cohort_inf'" "`baseyear'"
			}
			
						
			//In constructing the IGE copula, parents with income <= 0 were dropped. In other words,
			//parent-income columns correspond only to positive incomes. To adjust for this in our code, 
			//we drop all parents with <= 0 incomes; but keep a record of the fraction of rows dropped,
			//and bring them back in later while computing absolute mobility stats
			local focalCensus = 10*ceil(`cohort'/10)
			
			su income if income > 0 [fw = weights]
			local n_pos_income = r(sum_w)

			su income [fw = weights]
			local n_income = r(sum_w)
			
			local frac0_`cohort' =  1 - (`n_pos_income' / `n_income')
			
			//drop zero (and neg) incomes if the adjust_for_zeroes flag is set.
			if "`adjust_for_zeroes'" == "true" {
				
				if "`debug'" == "true"{
					//Save a copy of the dropped rows
					preserve
					keep if income <= 0
					save "par_sample_`cohort'_zeroes", replace
					restore
				}
				keep if income > 0
			}
						
			//Save a copy of the sample just before percentiles are computed
			if "`debug'" == "true"{
				save "par_sample_`cohort'", replace
			}
			
			//Compute income percentiles
			fastxtile percentile = income [fw=weights], n(100)
			collapse (min) par_inc_min = income, by(percentile)
			keep percentile par_inc_min
			
			//If cases where consequtive income cutoffs are identical (e.g. if ptile 65 == ptile 66)
			//we end up with missing percentiles using fastxtile. Fix this.
			//For all missing values, if _n = 2 or 1, set value to 0 else, set value to _N-1
			merge 1:1 percentile using "$absmob_indir/zero_incomes", nogen
			sort percentile
			replace par_inc = cond(mi(par_inc), cond(_n<=2, 0, par_inc[_n-1]), par_inc)			
			replace par_inc = round(par_inc)			
			gen cohort = `cohort'
			
			tempfile par_marginal_`cohort'
			save `par_marginal_`cohort'', replace

		} //cohort loop closes

		//combine all cohort-wise marginal incomes into a single file
		token `cohorts'
		use `par_marginal_`1'', clear
		macro shift	
		foreach cohort in `*' {
			append using `par_marginal_`cohort'' 
		}
		sort cohort percentile
		saveold "$absmob_outdir/par_marginal", replace version(12)

		//Save the fraction of parents with income <= 0 for each cohort		
		keep cohort
		duplicates drop
		gen double frac0 = .
		foreach cohort in `cohorts' {
				replace frac0 = `frac0_`cohort'' if cohort == `cohort'
		}
		saveold "$absmob_outdir/parents_frac0", replace version(12)
		
	} //compute_parents condition closes

	*************************************
	***				KIDS			  ***
	*************************************

	if "`compute_kids'" == "true" {	
			
		foreach cohort in `cohorts' {
			
			use "$absmob_indir/CPS_clean", clear

			local year = `cohort' + `kid_age'			
			keep if year == `year'
			keep if age == `kid_age' | age_spouse == `kid_age'
			
			if "`kid_gender'" == "male" keep if (sex == 1 & age ==`kid_age') | (sex_spouse == 1 & age_spouse == `kid_age')
			if "`kid_gender'" == "female" keep if (sex == 2 & age ==`kid_age') | (sex_spouse == 2 & age_spouse == `kid_age')
			if "`immigrants'" != "keep" & `year' >= 1994 drop if foreigner == 1
			
			//If age of both spouses == kid_age, then double the weight of that row
			expand 2 if (age==age_spouse)
									
			//Generate the "income" variable
			generate_income "`inc'" "cps" "`year'" "`kid_gender'" "`divby'" "`raiseby'"
			
			//compute income post-tax, if necessary
			if "`taxby'" == "mtr"{
				//income is already in terms taxyear
				compute_tax `year'
				replace income = income - ftaxmtr
			}
			
			//Scale income by inflator
			inflate_income "`inflator'" "$absmob_indir" "`baseyear'"
			
			//Save a copy of the sample just before percentiles are computed
			if "`debug'" == "true"{
				save "kid_sample_`cohort'", replace
			}

			//Compute income percentiles, weighted by wtsupp
			fastxtile percentile = income [pw=wtsupp], n(100)
			collapse (min) kid_inc_min = income, by(percentile)
			keep percentile kid_inc_min
			
			//Handle missing percentiles (see note in par_marginal construction)
			merge 1:1 percentile using "$absmob_indir/zero_incomes", nogen
			sort percentile
			replace kid_inc = cond(mi(kid_inc), cond(_n<=2, 0, kid_inc[_n-1]), kid_inc)							
			qui replace kid_inc = round(kid_inc)			
			gen cohort = `cohort'
			
			tempfile kid_marginal_`cohort'
			save `kid_marginal_`cohort''
			
		} //cohort loop closes

		//combine all cohort-wise marginal incomes into a single file
		token `cohorts'
		use `kid_marginal_`1'', clear
		macro shift		
		foreach cohort in `*' {
			append using `kid_marginal_`cohort'' 
		}
		sort cohort percentile
		saveold "$absmob_outdir/kid_marginal", version(12) replace		

	} //compute_children condition closes



	*************************************
	***			MOBILITY			  ***
	*************************************

	if "`compute_absmob'" == "true" {
	
		use "$absmob_indir/`copula'", clear	
		
		* If the copula file has no `cohort' variable, just append the copula to itself, for each cohort			
		capture confirm variable cohort
		
		if _rc != 0 {
			token `cohorts'
			gen cohort = `1'
			saveold "$absmob_outdir/`copula'_by_year", version(12) replace
			macro shift
			foreach cohort in `*' {
				use "$absmob_indir/`copula'", clear
				gen cohort = `cohort'
				append using "$absmob_outdir/`copula'_by_year"
				saveold "$absmob_outdir/`copula'_by_year", version(12) replace
			}
		}
		else{
			saveold "$absmob_outdir/`copula'_by_year", version(12) replace
		}
		
		
		use "$absmob_outdir/par_marginal", clear
		keep percentile cohort par_inc
		merge 1:1 percentile cohort using "$absmob_outdir/kid_marginal", assert(match) nogen
		rename percentile kid_bin
		merge 1:1 kid_bin cohort using "$absmob_outdir/`copula'_by_year", nogen
		rename kid_bin percentile

		gen double inflection = 0 // the kids' income rank at which kid income crosses par income
		gen double abs_mob = 0 // absolute mobility -- fraction of kids earning at least par income
			
		foreach c in `cohorts' {			
			
			preserve

			keep if cohort == `c'
			
			forvalues i = 1/100{
				
				**For parent income at the i'th percentile (i.e., parent_income[i]),
				**find the index 'j' at which kid_income[j] is at least as high as parent_income[i].
				**Interpolate linearly between `j' and `j+1' if kid_inc[j] < par_inc[i] < kid_inc[j+1]
				
				local parent_income = par_inc[`i']
				find_rank kid_inc, val(`parent_income')
				replace inflection = `r(index)' if _n == `i'
				local base = floor(inflection[`i'])
				local frac = inflection[`i'] - `base'
				
				if `base' == 0{
					**even the kid at the 0th percentile earns more than this parent
					replace abs_mob = 1 if _n == `i'
				}
				else if `base' == 100{
					**even the kid at the 100th ptile doesn't earn as much as this parent
					replace abs_mob = 0 if _n == `i'
				}
				else{
					local colsum = 0
					forvalues j = 1/`base'{
						local colsum = `colsum' + p`i'[`j']
					}
					local j = `base' + 1
					local colsum = `colsum' + (p`i'[`j'] * `frac')
					replace abs_mob = (1 - `colsum') if _n == `i'
				}
			}
			tempfile rank_`c'
			keep cohort percentile inflection abs_mob*
			save `rank_`c''
			restore
		}

		clear
		foreach c in `cohorts' {
			append using `rank_`c''
		}	
		if "`adjust_for_zeroes'" == "true"{			
			rename abs_mob abs_mob_pos_par
		}
		rename percentile parent_percentile
		saveold "$absmob_outdir/absmob", replace version(12)
	}
		
	capture log close //end logging

end

 
*************************************
***		  HELPER ROUTINES		  ***
*************************************

//A helper routine to parse parent options to absmob
capture program drop parse_parent_options
program define parse_parent_options, sclass
	syntax anything [, ///
		age_at_census(numlist asc integer min=2 max=2 >=0 <=100) ///
		age_at_childbirth(numlist asc integer min=2 max=2 >=0 <=100) ///
		GENder(string) REPtype(string)]
	
	local gender_opts male female all
	local reptype_opts richer older mother father
	
	//Assign default values to unspecified arguments
	if "`reptype'" == "" local reptype = "richer"
	if "`gender'" == "" local gender = "all"
	if "`age_at_census'" == "" local age_at_census 25 35
	if "`age_at_childbirth'" == "" local age_at_childbirth 16 45
	
	//check if `gender' is valid
	concat_list `gender_opts', addquotes
	local csl = r(concat_str)
	if !inlist("`gender'", "`csl'") {
		di as error "parents' gender can only be one of: `gender_opts'"
		exit 197
	}
	
	//check if `reptype' is valid
	concat_list `reptype_opts', addquotes
	local csl = r(concat_str)
	if !inlist("`reptype'", "`csl'") {
		di as error "parents' representative type can only be one of: `reptype_opts'"
		exit 197
	}
	
	token `age_at_census'
	sreturn local income_age_min = `1'
	sreturn local income_age_max = `2'
	
	token `age_at_childbirth'
	sreturn local parent_age_min = `1'
	sreturn local parent_age_max = `2'
	
	sreturn local par_gender `gender'
	sreturn local rep_parent `reptype'
	
end

//A helper routine to parse kid options to absmob
capture program drop parse_kid_options
program define parse_kid_options, sclass
	syntax anything [, ///
			AGE(integer 30) ///			
			GENder(string) ///
			IMMigrants(string)]
		
	local gender_opts male female all
	local immigrant_opts keep drop
	
	//Assign default values to unspecified arguments
	if "`age'" == "" local age = 30
	if "`gender'" == "" local gender = "all"
	if "`immigrants'" == "" local immigrants = "keep"
	
	//check if `gender' is valid
	concat_list `gender_opts', addquotes
	local csl = r(concat_str)
	if !inlist("`gender'", "`csl'") {
		di as error "kids' gender can only be one of: `gender_opts'"
		exit 197
	}
	
	concat_list `immigrant_opts', addquotes
	local csl = r(concat_str)
	if !inlist("`immigrants'", "`csl'") {
		di as error "immigrants can only be one of: `immigrant_opts'"
		exit 197
	}
	
	sreturn local immigrants = "`immigrants'"
	sreturn local kid_age `age'
	sreturn local kid_gender `gender'
	
end

//A helper routine to parse income options to absmob
capture program drop parse_income_options
program define parse_income_options, sclass
	syntax anything [, ///
		MEAsure(string) ///
		INFlator(string) ///
		BASEyear(integer 2014) ///
		DIVby(string) ///
		TAXby(string) ///
		RAISEby(integer 0) ///
		]
		
	local type_opts inctot faminc indinc incmix incwage inctotmx momfinc 
	local inf_opts cpirs ppi gdp pcepi cpi08 cpi10 cpi20 cpiu
	local div_opts famsize famsqrt spouses
	local tax_opts mtr taxsim
	
	//Assign default values to unspecified arguments
	if "`measure'" == "" local measure "incmix"
	if "`inflator'" == "" local inflator = "cpirs"
	
	//check if income `type' is valid
	concat_list `type_opts', addquotes
	local csl = r(concat_str)
	if !inlist("`measure'", "`csl'") {
		di as error "Income type can only be one of: `type_opts'"
		exit 197
	}
	
	//check if `inflator' is valid
	concat_list `inf_opts', addquotes
	local csl = r(concat_str)
	if !inlist("`inflator'", "`csl'") {
		di as error "Inflator can only be one of: `inf_opts'"
		exit 197
	}
	
	//check if `divby' is valid
	if "`divby'" != ""{
		concat_list `div_opts', addquotes
		local csl = r(concat_str)
		if !inlist("`divby'", "`csl'") {
			di as error "divby can only be one of: `div_opts'"
			exit 197
		}
	}
	
	//check if `taxby' is valid
	if "`taxby'" != ""{
		concat_list `tax_opts', addquotes
		local csl = r(concat_str)
		if !inlist("`taxby'", "`csl'") {
			di as error "taxby can only be one of: `div_opts'"
			exit 197
		}
	}
	//TODO: check if baseyear is valid
	
	sreturn local measure `measure'
	sreturn local inflator `inflator'
	sreturn local baseyear `baseyear'
	sreturn local divby `divby'
	sreturn local taxby `taxby'
	sreturn local raiseby `raiseby'
	
end

//A helper routine to parse the "compute" flag to absmob
capture program drop parse_cmd_options
program define parse_cmd_options, sclass	
	
	syntax namelist(min=2 max=2)	
	local verb_opts compute // Only "compute" is currently supported.
	local compute_opts par par_marginal kid kid_marginal mob mobility all
	
	tokenize `namelist'
	
	//Check if the suboption provided to 'absmob' is valid
	concat_list `verb_opts', addquotes
	local csl = r(concat_str)
	if !inlist("`1'", "`csl'") {
		di as error "absmob can only be followed by one of: `verb_opts'"
		exit 197
	}
		
	if "`1'" == "compute"{	
	
		//check if the suboption to `compute' is valid
		
		concat_list `compute_opts', addquotes
		local csl = r(concat_str)
		if !inlist("`2'", "`csl'") {
			di as error "compute can only be followed by one of: `compute_opts'"
			exit 197
		}	
		sreturn local compute `2'
	}
	
end

//A helper routine to parse dir options and set global paths
capture program drop parse_dir_options
program define parse_dir_options, sclass
	syntax anything [,INdir(string) OUTdir(string) WORKingdir(string)]
		
	if "`indir'" == "" {
		if "$absmob_indir" == "" {
			di as error "indir is required, unless the global absmob_indir is set"
			exit 197
		}
	}
	else{
		global absmob_indir = "`indir'"
	}
	
	if "`outdir'" == "" {
		if "$absmob_outdir" == "" {
			di as error "outdir is required, unless the global absmob_outdir is set"
			exit 197
		}
	}
	else{
		global absmob_outdir = "`outdir'"		
	}
	
	//Make the dir if it doesn't exist
	confirmdir "$absmob_outdir"
	if _rc > 0 { // _rc >0 if it doesn't exist
		mkdir "$absmob_outdir"
	}

	//Set workingdir to outdir unless otherwise specified
	if "`workingdir'" == "" {
		global absmob_workingdir "$absmob_outdir"
	}
	
end

//A helper routine to parse plot options
capture program drop parse_plot_options
program define parse_plot_options, sclass
	syntax anything [,indir(string) outdir(string) workingdir(string)]
end


//A program that concatenates a list into a delim-separated string
capture program drop concat_list
program define concat_list, rclass
	syntax namelist [, delim(string) addquotes]
	
	if "`delim'" == ""{
		local delim ","
	}
	
	tokenize `namelist'	
	
	if "`addquotes'" == "addquotes"{
		local str = `""`1'""'
	}
	else{
		local str `1'
	}
	macro shift
	
	while "`1'" != ""{
		if "`addquotes'" == "addquotes"{
			local str =  `"`str'"' + "`delim'" +  `""`1'""'
		}
		else{
			local str `str'`delim'`1'
		}
		macro shift
	}
	return local concat_str `str'
end


//A program that defines the "representative parent" for each set of parents
capture program drop generate_rep_id
program define generate_rep_id

	args rep_type par_gender_type census_year
		
	//overwrite rep_parent base on par_gender, if required.
	if "`par_gender_type'" == "male" {
		local rep_type = "father"
	}
	if "`par_gender_type'" == "female" {
		local rep_type = "mother"
	}
	
	capture drop rep_id
	capture drop rep_age
	capture drop rep_sex

	gen double rep_id = id
	
	if "`rep_type'" == "mother" {
		//Also sets rep_id to missing if it's a single father
		replace rep_id  = id_spouse  if sex != 2	
	}
	if "`rep_type'" == "father" {
		//Also sets rep_id to missing if it's a single mother
		replace rep_id  = id_spouse  if sex != 1 							
	}
	if "`rep_type'" == "older" {
		replace rep_id  = id_spouse  if !mi(age_spouse) & age_spouse > age				
	}
	if "`rep_type'" == "richer" {
		if `census_year' == 1940 {
			replace rep_id  = id_spouse  if incwage != . & incwage < incwage_spouse & id_spouse  != .
			replace rep_id  = id_spouse  if incwage == . & incwage_spouse != .	& id_spouse  != .
			replace rep_id  = id_spouse  if incwage_spouse !=. &  incwage == incwage_spouse & sex == 1 & id_spouse  != .
		}
		if `census_year' >  1940 {
			replace rep_id  = id_spouse  if inctot < inctot_spouse & id_spouse  != .
			replace rep_id  = id_spouse  if inctot == . & inctot_spouse != . & id_spouse  != .		
			replace rep_id  = id_spouse  if inctot_spouse !=. &  inctot == inctot_sp & sex == 1 & id_spouse  != .					
		}
		if `census_year' == 1950 {
			replace rep_id  = id_spouse  if inctot == . & inctot_spouse == . & age_spouse !=. & sex == 1
		}
	}			
	
	//Drop single mothers if par_gender = "father", etc.
	drop if mi(rep_id)
	gen rep_age = cond(rep_id == id, age, age_spouse)
	gen rep_sex = cond(rep_id == id, sex, sex_spouse)
end


//A program that defines income based on the income-type specified
capture program drop generate_income
program define generate_income

	args inc data_type year gender divby raiseby
	
	capture drop income
	gen income = .
		
	//create new (temporary) variables that will allow us to use rowtotal
	tempvar incwelfr incwelfr_spouse //incssi incssi_spouse
	gen `incwelfr' = -incwelfr
	gen `incwelfr_spouse' = -incwelfr_spouse
	if "`data_type'" == "census" { 
		//gen `incssi' = -incsupp
		//gen `incssi_spouse' = -incsupp_spouse 
	}
	if "`data_type'" == "cps" { 
		//gen `incssi' = -incssi
		//gen `incssi_spouse' = -incssi_spouse 
	}
	
	tempvar income1940 income1950 income1960 inctotOnly
		
	//egen `income1960' = rowtotal(inctot inctot_spouse `incwelfr' `incwelfr_spouse' `incssi' `incssi_spouse')
	egen `income1960' = rowtotal(inctot inctot_spouse `incwelfr' `incwelfr_spouse')
	egen `inctotOnly' = rowtotal(inctot inctot_spouse)
		
	if "`data_type'" == "census"{
		gen `income1950' = ftotinc		
		egen `income1940' = rowtotal(incwage incwage_spouse nonwage)	
	}
	else{ //type == cps. We don't even need this, since the CPS isn't available for these years
			gen `income1950' = ftotval
			egen `income1940' = rowtotal(incwage incwage_spouse)	
	}
		
	if "`inc'" == "incmix" | "`inc'" == "inctotmx" {
		replace income = `income1940' if year >= 1940 & year < 1950
		replace income = `income1950' if year >= 1950 & year < 1960
								
		//Uncomment the line below to use 1950 FTOTINC for 1940s income.
		//replace income = `income1950' if year >= 1940 & year < 1960
		replace income = `income1960' if year >= 1960		
	}
	
	if "`inc'" == "inctot" {
		replace income = `income1940' if year >= 1940 & year < 1950				
		replace income = `inctotOnly' if year >= 1950
	}
	
	if "`inc'" == "indinc" {
		if "`data_type'" == "census" & ("`gender'" == "male" | "`gender'" == "female")  {
			//foreach var in incwage inctot incwelfr incsupp {
			foreach var in incwage inctot {
				replace `var' = `var'_spouse if rep_sex != sex
			}	 
		}
		replace income = incwage if year >= 1940 & year < 1950
		replace income = inctot if year >=  1950 
		replace income = income - incwelfr if year >= 1968 & incwelfr != .
	  //replace income = income - incsupp if year >= 2000 & incsupp != .
		if "`data_type'" == "cps" { 
			if "`gender'"=="male"   local gdr 1
			if "`gender'"=="female" local gdr 2			
			replace income = inctot_spouse 			  if sex != `gdr' & sex_spouse == `gdr' & inctot_spouse   != .
			replace income = income - incwelfr_spouse if sex != `gdr' & sex_spouse == `gdr' & incwelfr_spouse != .
		  //replace income = income - incssi_spouse   if sex != `gdr' & sex_spouse == `gdr' & incssi_spouse   != .
		}
	}
	
	if "`inc'" == "momfinc" & "`gender'" == "female" {
		if "`data_type'" == "census" {
			replace income = ftotinc - inctot 		 if sex_spouse == 2 & inctot 	    != .
			replace income = ftotinc - inctot_spouse if sex_spouse == 1 & inctot_spouse != .
		}
		if "`data_type'" == "cps" {
			replace income = ftotval - inctot 		 if sex_spouse == 2 & inctot 	    != .
			replace income = ftotval - inctot_spouse if sex_spouse == 1 & inctot_spouse != .
		}
	}
	
	if "`inc'" == "faminc" {
		replace income = `income1940' if year >= 1940 & year < 1950		
		replace income = `income1950' if year >= 1950
	}
	
	if "`inc'" == "incwage" {
		replace income = `income1940' 		
	}

	if "`inc'" == "inctotmx"{
		replace income = `inctotOnly' if year >= 1960
	}
	
	** Raise Kids Income by X **
	if "`raiseby'" != "0" & "`data_type'" == "cps" {
		replace income = income + `raiseby'
	}
	
	**Normalize income for various factors
	if "`divby'" == "famsize" {
		drop if mi(famsize) | famsize == 0
		replace income  = income/famsize
	}
	if "`divby'" == "famsqrt" {
		drop if mi(famsize) | famsize == 0
		replace income  = income/sqrt(famsize)
	}
	if "`divby'" == "spouses" {
		replace income = income/2 if id_spouse != .
	}	
	
	drop if income == . 
	
end

//A program to adjust incomes to a particular year by a given index
//Assumes the variable `income' and `year' are defined
capture program drop inflate_income
program define inflate_income, rclass

	args index inputfolder baseyear
	
	preserve	
	use "`inputfolder'/`index'", clear
	sort year
	keep year rate
	tempfile inflation
	save    `inflation', replace
	keep if year == `baseyear'
	local inflation_rate = rate[1]
	restore
	
	replace year = year - 1 //income refers to previous calendar year
	merge m:1 year using `inflation', assert(match using) keep(match master)
	drop _merge
	replace year = year + 1
	replace rate = rate / `inflation_rate'
	replace income = income/rate
	drop rate
	drop if mi(income) //TODO: Is this necessary?
end


//A program to scale incomes to a particular year by an index.
//The difference from the previous program is that the years are constants, not variables
capture program drop inflate_income_const 
program define inflate_income_const, rclass
args index_file income_var from_year to_year
	
	preserve
	use "`index_file'", clear
	keep year rate
	gen from_rate = rate if year == `from_year'
	su from_rate
	local from_rate = `r(max)'
	gen to_rate = rate if year == `to_year'
	su to_rate
	local to_rate = `r(max)'
	restore
	replace `income_var' = (`income_var' * `to_rate') / `from_rate'
	
end

//return the smallest index i such that A[i] < value
program define find_rank, rclass

	syntax varlist(min=1 max=1), VALue(real)
	local varname `varlist' //because only one value is allowed
	
	sort `varname'
	
	if `value' < `varname'[1] {
		return local index 0
		exit
	}
	
	local j = 1
	while `j' <= _N & `varname'[`j'] < `value'{
		local j = `j' + 1
	}
	
	if `j' > _N{
		return local index _N
		exit
	}
	
	if `j' <= _N & `j' > 1{
		local j = (`j'-1) + ((`value' - `varname'[`j'-1]) / (`varname'[`j'] - `varname'[`j' - 1]))
	}
	return local index `j'
	
end


**A program to read in income data and compute tax liabilities
capture program drop compute_tax
program define compute_tax, rclass
	args taxyear
	
	//The caller is expected to handle all incomes <= 0
	//assert income > 0
	
	//TODO: un-hardcode this?
	local mtr_file = "$absmob_indir/nominal_tax_rates"
	local mtr_exemption_file = "$absmob_indir/exemptions"
	
	**Type of filing: Married filing jointly; Married filing separately; Single; Head-of-household
	local ftypelist jnt sep sin hhh
	
	*Convert IPUMS marriage status to tax-filing status
	
	** IPUMS flags for marital status
	** 1 - Married, spouse present 
	** 2 - Married, spouse absent
	** 3 - Separated
	** 4 - Divorced
	** 5 - Widowed
	** 6 - Never married/single
	
	*We assume any married couple is filing jointly
	gen ftype = "jnt" if marst == 1 | marst == 2
	
	replace ftype = "sep" if marst == 3 //"sep" - Married, filing separately
	replace ftype = "sin" if marst > 3  //"sin" - Single		
	
	*We assume any single parent is filing as Head-of-household;
	*and we use nchild to approximate no. of children under 18 (depchild)		
	replace ftype = "hhh" if marst > 2 & nchild > 0
		
	** Get cutoffs into local lists
	preserve
	
	use "`mtr_file'", clear
	keep if year == `taxyear'
	drop year
	tempfile mtr_file_yr
	save `mtr_file_yr'
	
	foreach ft in `ftypelist'{
		use `mtr_file_yr', clear
		keep if ftype == "`ft'"	
		keep cutoff
		sort cutoff
		local cp_`taxyear'_`ft'
		count
		foreach i of numlist 1/`r(N)'{
			local cp_i = cutoff[`i']
			local cp_`taxyear'_`ft' `cp_`taxyear'_`ft'' `cp_i'
		}
		//append "infinity" -- needed for egen cut() to work properly
		local inf = cutoff[_N] + 1
		local cp_`taxyear'_`ft' `cp_`taxyear'_`ft'' `inf'		
	}
	restore
	
	** Get exemptions into local lists
	preserve
	use "`mtr_exemption_file'", clear
	keep if year == `taxyear'
	foreach ft in `ftypelist'{
		//a local that stores the exemption for a given (taxyear, filing-type)
		local ex_`taxyear'_`ft' = `ft'[1]
	}
	//a local that stores the exemption per dependent, for a given taxyear
	local ex_`taxyear'_dep = dep[1]
	restore
	
	**Compute taxable income
	gen taxable_inc = income
	foreach ft in `ftypelist'{
		replace taxable_inc = taxable_inc - `ex_`taxyear'_`ft'' if ftype == "`ft'"
		replace taxable_inc = taxable_inc - nchild * `ex_`taxyear'_dep' if ftype == "`ft'" & !mi(nchild)
		replace taxable_inc = 0 if ftype == "`ft'" & taxable_inc < 0
	}
	*calculate which bracket each income falls in (the highest cutoff below this income)
	
	foreach ft in `ftypelist'{
		//Use cutoffs to determine which bracket each income falls in
		preserve
		keep if ftype == "`ft'"
		egen cutoff = cut(taxable_inc), at(`cp_`taxyear'_`ft'')
		gen marginal_inc = taxable_inc - cutoff
		tempfile inc_`taxyear'_`ft'
		save `inc_`taxyear'_`ft''
		restore
	}
	
	//Append all temporary files into a single file
	foreach ft in `ftypelist'{	
		append using `inc_`taxyear'_`ft''
	}
	
	//Compute the actual tax liability
	merge m:1 ftype cutoff using `mtr_file_yr', nogen keep(match)
	gen double ftaxmtr = round((mtr*marginal_inc)/100 + min_tax)
	replace ftaxmtr = 0 if income <= 0
	
	save "mtr_sample_`taxyear'", replace
	if "`debug'" != "true"{
		drop mtr cutoff ftype min_tax marginal_inc 
	}
	
end


//Confirm if a directory exists.

*! comfirmdir Version 1.1 dan_blanchette@unc.edu 22Jan2009
* Center of Entrepreneurship and Innovation Duke University's Fuqua School of Business
* confirmdir Version 1.1 dan_blanchette@unc.edu  17Jan2008
* research computing, unc-ch
*  - made it handle long directory names
** confirmdir Version 1.0 dan_blanchette@unc.edu  05Oct2003
** the carolina population center, unc-ch

program define confirmdir, rclass
 version 8
 
 local cwd `"`c(pwd)'"'
 quietly capture cd `"`1'"'
 local confirmdir=_rc 
 quietly cd `"`cwd'"'
 return local confirmdir `"`confirmdir'"'
 
end 
