/* Runs the code to produce the main figures */

* Edit these globals to reflect your local paths 
global absmob_indir  "" //The path to the cleaned data directory (see input_clean in README)
global absmob_resultsdir "" //The path to a local folder that will contain all the resulting datasets
global absmob_sourcesdir "" //The path to the folder containing the downloaded source code (e.g. sources.do)

discard
adopath + "$absmob_sourcesdir"
clear
set more off

global image_suffix png
global num_levels 500

log using "$absmob_resultsdir/Baseline/draw_figs_log", replace

***************************************************
/* Fig 1a: Abs. mobility by parent percentile, 1 per decade */
***************************************************

use "$absmob_resultsdir/Baseline/figure_input_data", clear

replace abs_mob_pos = 100*abs_mob_pos

twoway (line abs_mob_pos parent_percentile if cohort == 1940, lp(solid)) || ///
	   (line abs_mob_pos parent_percentile if cohort == 1950, lp(solid)) || ///
       (line abs_mob_pos parent_percentile if cohort == 1960, lp(solid)) || ///
	   (line abs_mob_pos parent_percentile if cohort == 1970, lp(solid)) || ///
	   (line abs_mob_pos parent_percentile if cohort == 1980, lp(solid)) ||, ///
	title("Figure 1a") ///
	xlabel(0(20)100, grid) ///
	graphregion(lcolor(none) ilcolor(none) fcolor(white)) ///
	ytitle("Pct. of Children Earning more than their Parents") ///
	ylabel(, angle(0)) /// 
	xtitle("Parent Income Percentile (conditional on positive income)") ///
	text(95 84 "1940") ///
	text(76 84 "1950") ///
	text(57 84 "1960") ///
	text(42 84 "1970") ///
	text(31 84 "1980") ///
	legend(off)
	
graph export "$absmob_resultsdir/Baseline/Fig1a.${image_suffix}", replace

***************************************************
/***** Fig 1b: Mean absolute mobility by cohort **/
***************************************************

use "$absmob_resultsdir/Baseline/figure_input_data", clear
sort cohort parent_percentile
by cohort: egen cohort_means_pos = mean(abs_mob_pos_par)
gen cohort_means = (1-frac0)*cohort_means_pos + frac0
sort cohort parent_percentile
replace cohort_means = 100*cohort_means
collapse (mean) cohort_means, by(cohort)

twoway scatter cohort_means cohort, ///
	connect(direct) lcolor(navy) msymbol(circle) ///
	xtitle("Child's Birth Cohort") ///
	ylabel(50(10)100, angle(0)) ///
	ytitle("Pct. of Children Earning more than their Parents") ///
	title("Figure 1b") ///
	graphregion(lcolor(none) ilcolor(none) fcolor(white)) 
graph export "$absmob_resultsdir/Baseline/Fig1b.${image_suffix}", replace


***************************************************
/***** Fig 2a: Robustness to FOSD Copulas ********/
***************************************************

* NOTE THIS GETS SENT TO MATLAB TO MAKE THE BOUNDS -- PRODUCES out.txt that has lower and upper bounds (FOSD) in v2 and v3
insheet using "$absmob_resultsdir/matlab/out.txt", clear
rename v1 cohort
rename v2 lb_fosd_temp
rename v3 ub_fosd_temp
merge 1:1 cohort using "$absmob_resultsdir/Baseline/parents_frac0", nogen // add in zero parent incomes
gen lb_fosd = (1-frac0)*lb_fosd_temp + frac0
gen ub_fosd = (1-frac0)*ub_fosd_temp + frac0
label var lb_fosd "Lower Bound"
label var ub_fosd "Upper Bound"
tempfile temp
save `temp' , replace
clear

// Baseline series
use "$absmob_resultsdir/Baseline/absmob", clear
merge m:1 cohort using "$absmob_resultsdir/Baseline/parents_frac0", nogen
sort cohort parent_percentile
by cohort: egen cohort_means_pos = mean(abs_mob_pos_par)
gen cohort_means = (1-frac0)*cohort_means_pos + frac0
keep cohort_means cohort
label var cohort_means "Baseline"
duplicates drop 
merge 1:1 cohort using `temp' , nogen

replace cohort_means = 100*cohort_means
replace lb_fosd = 100*lb_fosd
replace ub_fosd= 100*ub_fosd
sort cohort

tw (scatter cohort_means cohort, connect(direct) lcolor(navy) mcolor(navy) msymbol(circle)) ///
   (line lb_fosd 	     cohort, connect(direct) lcolor(navy) lpattern(dash) lw(medthick)) ///
   (line ub_fosd 		 cohort, connect(direct) lcolor(navy) lpattern(dash) lw(medthick)) , ///
	xtitle("Child's Birth Cohort") ///
	ytitle("Pct. of Children Earning more than their Parents") ///
	title("Figure 2a") ///
	graphregion(lcolor(none) ilcolor(none) fcolor(white)) ///
	legend(off) ylab(,angle(horizontal)) ///
	xline(1970.5, lcolor(maroon) lp(-)) ///
	text(28 1980 "Lower Bound" ///
		97 1980 "Upper Bound" ///
		76 1973 "Copula" ///
		72 1974 "Observed" ///
		43 1980 "Baseline Estimates") 
graph export "$absmob_resultsdir/Baseline/Fig2a.${image_suffix}", replace


***************************************************
/***** Fig 2b: 1940 Income Distributions **************/
***************************************************

// 1940 cohort
use "$absmob_resultsdir/Baseline/figure_input_data", clear
di in red "Required Rank to Beat Parents at p80, 1940 birth cohort"
list parent_percentile cohort inflection if parent_percentile==80 & cohort==1940 

use "$absmob_resultsdir/Baseline/kid_marginal", clear
merge 1:1 percentile cohort using "$absmob_resultsdir/Baseline/par_marginal", nogen

list par_inc if percentile==80 & cohort==1940

tw (kdensity par_inc if cohort==1940, lw(thick) bw(6000)) ///
   (kdensity kid_inc if cohort==1940, lw(thick)) ///
	if par_inc<=200000 & kid_inc<=200000, ///
	ytitle("Density") ylab(none) ///
	xtitle("Income (Measured in Real 2014$)") xline(27000, lp(-) lc(gs0)) ///
	xlab(0 27000 "27k" 50000 "50k" 100000 "100k" 150000 "150k") ///
	title("Figure 2b") ///
	text(.000011 32000  "14th percentile of children's distribution" ///
		 .000023 65000  "80th percentile of parents distribution" ///
		 .000003 60000  "Parents" ///
		 .000003 130000 "Children") ///
	legend(off)
		
graph export "$absmob_resultsdir/Baseline/Fig2b.${image_suffix}", replace


***************************************************
/***** Fig 2c: 1980 Income Distributions **************/
***************************************************
// 1980 cohort
use "$absmob_resultsdir/Baseline/figure_input_data", clear
di in red "Required Rank to Beat Parents at p80, 1980 birth cohort"
list parent_percentile cohort inflection if parent_percentile==80 & cohort==1980 
// current point (80,74)

use "$absmob_resultsdir/Baseline/kid_marginal", clear
merge 1:1 percentile cohort using "$absmob_resultsdir/Baseline/par_marginal", nogen

list par_inc if percentile==80 & cohort==1980

tw (kdensity par_inc if cohort==1980, lw(thick) bw(8000)) ///
	(kdensity kid_inc if cohort==1980, lw(thick)) ///
	if par_inc<=200000 & kid_inc<=200000, ///
	ytitle("Density") ylab(none) ///
	xtitle("Income (Measured in Real 2014$)") xline(80000, lp(-) lc(gs0)) ///
	xlab(80000, add) ///
	xlab(0 50000 "50k" 80000 "80k" 100000 "100k" 150000 "150k") ///
	title("Figure 2c") ///
	text(.000004  40000  "74th percentile of children's distribution" ///
		 .000007  140000 "80th percentile of parents distribution" ///
		 .0000015 100000 "Parents" ///
		 .0000015 185000 "Children") ///
	legend(off)
		
graph export "$absmob_resultsdir/Baseline/Fig2c.${image_suffix}", replace
	
	
****************************************************************
/***** Fig 2d: Needed Rank 1940 & 1980 Cohorts with heatmap***/
***************************************************************

// import copula
use "$absmob_indir/copula_base_adjusted", clear
ren p* density*
reshape long density, i( kid_bin ) j(par_fam_bin)
label variable kid_bin "Child Income Percentile"
label variable par_fam_bin "Parent Income Percentile"
tempfile copula
save `copula'

// minimum child rank to earn more than parent
use "$absmob_resultsdir/Baseline/figure_input_data", clear
drop abs_mob frac0
rename parent_percentile par_fam_bin
reshape wide inflection, i(par_fam_bin) j(cohort)
merge 1:m par_fam_bin using `copula', nogen

global start = 240 
global color_list ""
global label_list ""
sort density
gen density_rank = _n
global label_spacing = round($num_levels/10)
forvalues i=1/$num_levels {
	if mod(`i',$label_spacing) == 0 {
		qui sum density if density_rank <= `i' * _N / $num_levels
		local label_number = floor(`i' * _N / $num_levels)
		local label_value = strofreal(`r(max)',"%5.3f")
		global label_list = `"${label_list} `label_number' "`label_value'""'
	}
	local degree = $start - (`i' * $start / $num_levels)
	local saturation = `i' * 1 / $num_levels
	global color_list `"${color_list} "hsv `degree' `saturation' 1""'
}

twoway (contour density_rank kid_bin par_fam_bin, ///
	heatmap interp(none) levels(${num_levels}) crule(lin) scolor(white) ecolor(orange_red)) ///
	(line inflection1940 par_fam_bin if kid_bin==1, sort(par_fam_bin) c(l) lw(medthick) lpattern(solid) lc(gs0)) ///
	(line inflection1980 par_fam_bin if kid_bin==1, sort(par_fam_bin) c(l) lw(medthick) lpattern(solid) lc(gs0)) ///
	(scatteri 13.9 80 (11) "(80,14)", m(circle) mc(gs0) msize(medium) mlabsize(medium) mlabc(gs0) mlabgap(*2)) ///
	(scatteri 73.8 80 (11) "(80,74)", m(circle) mc(gs0) msize(medium) mlabsize(medium) mlabc(gs0) mlabgap(*2)), ///
	text(43 25 "1940" ///
		9 25 "1980", size(*1.2)) ///
	xtitle("Parent Income Percentile") ytitle("Child Income Percentile") ///
	title("Figure 2d") ///
	graphregion(color(white)) ///
	ylabel(, angle(0)) ///
	xlab( ,grid) ///
	legend(off) ///
	clegend(off)
graph export "$absmob_resultsdir/Baseline/Fig2d.png", replace


/***** Robustness  **********/
* robustness.do runs the code & saves the data for robustness checks

***************************************************
/***** Fig 3a: Robustness to Deflators  **********/
***************************************************

// PPI
use "$absmob_resultsdir/Robustness/PPI/absmob", clear
merge m:1 cohort using "$absmob_resultsdir/Robustness/PPI/parents_frac0", nogen 
bysort cohort: egen cohort_means_pos_ppi = mean(abs_mob_pos_par)
gen cohort_means_ppi = (1-frac0)*cohort_means_pos_ppi + frac0
tempfile absmob_ppi 
save `absmob_ppi'

// PCEPI
use "$absmob_resultsdir/Robustness/PCEPI/absmob", clear
merge m:1 cohort using "$absmob_resultsdir/Robustness/PCEPI/parents_frac0", nogen 
bysort cohort: egen cohort_means_pos_pcepi = mean(abs_mob_pos_par)
gen cohort_means_pcepi = (1-frac0)*cohort_means_pos_pcepi + frac0
tempfile absmob_pcepi 
save `absmob_pcepi'

// CPI minus 0.8pp
use "$absmob_resultsdir/Robustness/CPI minus .8/absmob", clear
merge m:1 cohort using "$absmob_resultsdir/Robustness/CPI minus .8/parents_frac0", nogen 
bysort cohort: egen cohort_means_pos_cpi08 = mean(abs_mob_pos_par)
gen cohort_means_cpi08 = (1-frac0)*cohort_means_pos_cpi08 + frac0
tempfile absmob_cpi08 
save `absmob_cpi08'

// Baseline
use "$absmob_resultsdir/Baseline/figure_input_data", clear
bysort cohort: egen cohort_means_pos_cpirs = mean(abs_mob_pos_par)
gen cohort_means_cpirs = (1-frac0)*cohort_means_pos_cpirs + frac0

merge 1:1  cohort parent_percentile using `absmob_ppi', nogen	
merge 1:1  cohort parent_percentile using `absmob_pcepi', nogen	
merge 1:1  cohort parent_percentile using `absmob_cpi08', nogen	

replace cohort_means_cpirs = 100*cohort_means_cpirs
replace cohort_means_ppi   = 100*cohort_means_ppi
replace cohort_means_pcepi = 100*cohort_means_pcepi
replace cohort_means_cpi08 = 100*cohort_means_cpi08

collapse (mean) cohort_means_*, by(cohort)

twoway (scatter cohort_means_cpirs cohort, connect(direct) msymbol(circle)   col(navy)) ///
	   (scatter cohort_means_ppi   cohort, connect(direct) msymbol(triangle) col(forest_green)) ///
	   (scatter cohort_means_pcepi cohort, connect(direct) msymbol(circle)   col(dkorange) mfc(none)) ///
	   (scatter cohort_means_cpi08 cohort, connect(direct) msymbol(square)   col(erose) mfc(none)), ///
		graphregion(lcolor(none) ilcolor(none) fcolor(white)) ///
		title("Figure 3a") ///
		xtitle("Child's Birth Cohort") ///
		ytitle("Pct. of Children Earning more than their Parents") ///
		ylabel(50(10)100, angle(0)) ///
		yscale(range(50 100)) ///
		legend(lab(1 "Baseline: CPI-U-RS") ///
			   lab(2 "PPI") ///
			   lab(3 "PCEPI") ///
		       lab(4 "CPI-U-RS minus 0.8%") pos(1) ring(0) c(1) order(1 4 3 2) region(color(none)))
graph export "$absmob_resultsdir/Baseline/Fig3a.${image_suffix}", replace


***************************************************
/***** Fig 3b: Robustness to Taxes & Transfers ***/
***************************************************

// Baseline
use "$absmob_resultsdir/Baseline/figure_input_data", clear
rename (inflection abs_mob frac0) (base_inflection base_absmob_pos_par base_frac0)
gen base_cohort_means = (1-base_frac0)*base_absmob_pos_par+base_frac0

// Taxes
merge 1:1 parent_percentile cohort using "$absmob_resultsdir/Robustness/taxes/absmob" , nogen
merge m:1 cohort using "$absmob_resultsdir/Robustness/taxes/parents_frac0.dta", nogen
rename (abs_mob_pos_par inflection frac0) (tax_absmob_pos_par tax_inflection tax_frac0)
gen tax_cohort_means = (1-tax_frac0)*tax_absmob_pos_par+tax_frac0

// Taxes and transfers: we are not able to share the data used for this series, and hence this series cannot be replicated here

replace base_cohort_means = 100*base_cohort_means
replace tax_cohort_means = 100*tax_cohort_means

* collapse income pctiles
collapse (mean) *cohort_means, by(cohort)

tw (scatter base_ cohort, connect(line) msymbol(circle) color(navy)) ///
   (scatter tax_  cohort, connect(line) msymbol(circle) color(maroon) mfc(none)), ///
	title("Figure 3b") ///
	ytitle("Pct. of Children Earning more than their Parents") ///
	xtitle("Child's Birth Cohort") ///
	yscale(range(50 100)) ylab(50(10)100, angle(horizontal)) ///
	legend(lab(1 "Baseline: Pre-Tax Income") lab(2 "Including Taxes") ///
		order(1 2) ring(0) pos(1) c(1) region(fcolor(none)))
		
graph export "$absmob_resultsdir/Baseline/Fig3b.$image_suffix", replace


***************************************************
/***** Fig 3c: Robustness to Life-Cycle **********/
***************************************************

// 40-year-olds
use "$absmob_resultsdir/Robustness/40 vs 40/absmob", clear
merge m:1 cohort using "$absmob_resultsdir/Robustness/40 vs 40/parents_frac0", nogen 
bysort cohort: egen cohort_means_pos_4040 = mean(abs_mob_pos_par)
gen cohort_means_4040 = (1-frac0)*cohort_means_pos_4040 + frac0
tempfile absmob_4040 
save `absmob_4040'

// Baseline
use "$absmob_resultsdir/Baseline/figure_input_data", clear
bysort cohort: egen cohort_means_pos_30 = mean(abs_mob_pos_par)
gen cohort_means_30 = (1-frac0)*cohort_means_pos_30 + frac0
merge 1:1 cohort parent_percentile using `absmob_4040', nogen
replace cohort_means_30   = 100*cohort_means_30
replace cohort_means_4040 = 100*cohort_means_4040

collapse (mean) cohort_means_*, by(cohort)

tw (scatter cohort_means_30   cohort, connect(direct) msymbol(circle)) ///
   (scatter cohort_means_4040 cohort, connect(direct) msymbol(triangle)) , ///
	title("Figure 3c") ///
	xtitle("Child's Birth Cohort") ///
	ytitle("Pct. of Children Earning more than their Parents") ///
	ylabel(50(10)100, angle(0)) ///
	graphregion(lcolor(none) ilcolor(none) fcolor(white)) ///
	legend(lab(1 "Baseline: Children Age 30, Parents 25-35") ///
		   lab(2 "Children Age 40, Parents 35-45") ///
		   c(1) pos(1) ring(0) region(color(none)))
		   
graph export "$absmob_resultsdir/Baseline/Fig3c.${image_suffix}", replace


***************************************************
/***** Fig 3d: Robustness to Individual Income (Fathers) ***/
***************************************************

// Father-Son Individual Incomes
use "$absmob_resultsdir/Robustness/Sons/Fathers/Individual Income/absmob", clear
merge m:1 cohort using "$absmob_resultsdir/Robustness/Sons/Fathers/Individual Income/parents_frac0", nogen 
bysort cohort: egen cohort_means_pos_men = mean(abs_mob_pos_par)
gen cohort_means_men = (1-frac0)*cohort_means_pos_men + frac0
tempfile absmob_men
save `absmob_men'

// Sqrt of Family Size
use "$absmob_resultsdir/Robustness/Family Size/absmob", clear
merge m:1 cohort using "$absmob_resultsdir/Robustness/Family Size/parents_frac0", nogen 
bysort cohort: egen cohort_means_pos_famsqrt = mean(abs_mob_pos_par)
gen cohort_means_famsqrt = (1-frac0)*cohort_means_pos_famsqrt + frac0
tempfile absmob_familysqrt 
save `absmob_familysqrt'

// Baseline
use "$absmob_resultsdir/Baseline/figure_input_data", clear
bysort cohort: egen cohort_means_pos_base = mean(abs_mob_pos_par)
gen cohort_means_base = (1-frac0)*cohort_means_pos_base + frac0
replace cohort_means_base  = 100*cohort_means_base
merge 1:1 cohort parent_percentile using `absmob_men', nogen
replace cohort_means_men   = 100*cohort_means_men
merge 1:1 cohort parent_percentile using `absmob_familysqrt', nogen
replace cohort_means_famsqrt = 100*cohort_means_famsqrt

collapse (mean) cohort_means_*, by(cohort)

tw (scatter cohort_means_base    cohort, connect(direct) msymbol(circle)) ///
   (scatter cohort_means_men     cohort, connect(direct) msymbol(triangle)) ///
   (scatter cohort_means_famsqrt cohort, connect(direct) msymbol(circle) mfc(none)), ///
	title("Figure 3d") ///
	xtitle("Child's Birth Cohort") ///
	ytitle("Pct. of Children Earning more than their Parents") ///
	ylabel(40(10)100, angle(0)) ///
	graphregion(lcolor(none) ilcolor(none) fcolor(white)) ///
	legend(lab(1 "Baseline - Family Income, All Children") ///
		   lab(2 "Individual Income, Fathers-Sons") ///
		   lab(3 "Divide by Sqrt(Family Size)") ///
		   pos(1) region(color(none)) c(1) ring(0) order(1 3 2))
		   
graph export "$absmob_resultsdir/Baseline/Fig3d.${image_suffix}", replace


***************************************************
* Fig 4a: Declines by State: OH, MA, MI, NY
***************************************************

use "$absmob_resultsdir/Robustness/States/state_rescaled_means", clear
replace cohort_mean_state_rescale = 100*cohort_mean_state_rescale

tw (scatter cohort_means cohort if state == 39, connect(direct) msymbol(circle)   mcolor(navy) 		   lcolor(navy)) ///
   (scatter cohort_means cohort if state == 25, connect(direct) msymbol(circle)   mcolor(erose) 	   lcolor(erose) mfc(none)) ///
   (scatter cohort_means cohort if state == 26, connect(direct) msymbol(triangle) mcolor(forest_green) lcolor(forest_green)) ///
   (scatter cohort_means cohort if state == 36, connect(direct) msymbol(square)   mcolor(dkorange) 	   lcolor(dkorange) mfc(none)), ///
	xtitle("Child's Birth Cohort") ///
	ylabel(40(10)100, angle(0)) ///
	ytitle("Pct. of Children Earning more than their Parents") ///
	title("Figure 4a") ///
	graphregion(lcolor(none) ilcolor(none) fcolor(white)) ///
	legend(lab(1 "Ohio") ///
		   lab(2 "Massachusetts") ///
		   lab(3 "Michigan") ///
		   lab(4 "New York") pos(2) ring(0) c(1) order(2 4 1 3) region(color(none)))		   
graph export "$absmob_resultsdir/Baseline/Fig4a.${image_suffix}", replace


***************************************************
* Fig 4b: Declines by State: Map
***************************************************

use "$absmob_resultsdir/Robustness/States/state_rescaled_means", clear
replace cohort_mean_state_rescale = 100*cohort_mean_state_rescale
collapse (first) abs40=cohort_mean_state_rescale (last) abs80=cohort_mean_state_rescale, by(state state_name)
gen diff = abs(abs80 - abs40) 
drop if mi(diff)
order state abs40 abs80 diff state_name
rename state statefips

ssc install spmap
ssc install maptile
maptile_install using "http://files.michaelstepner.com/geo_state.zip"
maptile diff, geography(state) geoid(statefips) fc(OrRd) ///
	twopt(title("Decline in Absolute Mobility, 1940 to 1980 cohorts") ///
	note("Note: With 1940 1% sample, AK, DE, HI, NV and WY start in 1960.")) ///
	savegraph("$absmob_resultsdir/Baseline/Fig4b.${image_suffix}") replace	
	
	
***************************************************
* Fig 5a: Counterfactuals 
***************************************************

use "$absmob_resultsdir/Baseline/counterfactuals" , clear
g actual1980 = abs_mob_1980obs
g actual1940 = abs_mob_1940obs
g hh1980 = abs_mob_1980hh
g ll1980 = abs_mob_1980ll
g perf1980 = abs_mob_1980mob

* mean abs mob. per counterfactual
preserve
collapse (mean) actual* hh ll perf
g id = _n
reshape long actual hh ll perf, i(id) j(cohort)
drop id
merge 1:1 cohort using "$absmob_resultsdir/Baseline/parents_frac0", nogen keep(master match)
g cohort_means_obs = (1-frac0)*actual+frac0
g cohort_means_hh = (1-frac0)*hh+frac0
g cohort_means_ll = (1-frac0)*ll+frac0
g cohort_means_perf = (1-frac0)*perf+frac0

replace cohort_means_perf = 100*cohort_means_perf
replace cohort_means_obs = 100*cohort_means_obs
replace cohort_means_hh = 100*cohort_means_hh
replace cohort_means_ll = 100*cohort_means_ll

sum cohort_means_perf if cohort==1980
di in red r(mean)
sum cohort_means_obs if cohort==1980
local act80_mean : di %2.1f r(mean)
sum cohort_means_obs if cohort==1940
local act40_mean : di %2.1f r(mean)
sum cohort_means_hh if cohort==1980
local hh_mean : di %2.1f r(mean)
sum cohort_means_ll if cohort==1980
local ll_mean : di %2.1f r(mean)

di in red "How much of the decline does high growth make up for:"
di in red (`hh_mean'-`act80_mean')/(`act40_mean'-`act80_mean')


di in red "How much of the decline does equitable distributional growth make up for:"
di in red (`ll_mean'-`act80_mean')/(`act40_mean'-`act80_mean')
restore

replace actual1940 = 100*actual1940
replace actual1980 = 100*actual1980
replace ll = 100*ll
replace hh = 100*hh

twoway (line actual1940 actual1980 parent_percentile) ///
	(line ll parent_percentile, lp(dash)) ///
	(line hh parent_percentile, lp(shortdash_dot)), ///
	xlabel(0(20)100, grid) ///
	ytitle("Pct. of Children Earning more than their Parents") ///
	ylabel(, angle(0)) /// 
	xtitle("Parent Income Percentile (conditional on positive income)") xlabel(, grid) ///
	title("Figure 5a") ///
	text(37 50 "Mean AM:`act80_mean'%" ///
		55 50 "Mean AM:`hh_mean'%" ///
		77 50 "Mean AM:`ll_mean'%" ///
		97 50 "Mean AM:`act40_mean'%" , size(small)) ///
	text(95 85 "1940 Empirical", size(small)) ///
	text(27 85 "1980 Empirical", size(small)) ///
	legend(order(3 4) label(4 "1940 GDP/family growth rate (2.5%), 1980 income shares") ///
		label(3 "1980 GDP/family growth rate (1.5%), 1940 income shares") ///
		pos(7) ring(0) region(color(none)) c(1) size(small))				
graph export "$absmob_resultsdir/Baseline/Fig5a.${image_suffix}" , replace


/**********************************************************
* Figure 5b: Counterfactual growth rates
*************************************************************/

use "$absmob_resultsdir/Baseline/counterfactuals_summary", clear
foreach var of varlist abs_mob* {
	replace `var'=100*`var'
	}
sum abs_mob_1940obs, mean
local am40 = r(mean)
sum abs_mob_1980obs, mean
local am80 = r(mean)

keep abs_mob_1980gr*
g id = _n
reshape long abs_mob_1980gr, i(id) j(growth)
tw line abs_mob growth, ///
	title("Figure 5b") ///
	ytitle("Pct. of Children Earning more than their Parents") ///
	xtitle("Real GDP/Family Growth Rate (%)") ///
	yline(`am40', lp(-)) text(94 5 "1940 Empirical") ///
	yline(`am80', lp(-)) text(52 5 "1980 Empirical") ///
	yscale(range(40 100)) ylab(40(10)100, angle(0))

graph export "$dropbox/absolute_mobility/results/Baseline/Fig5b.${image_suffix}", replace


cap log close
