/* Computes parents' marginal income distributions from CPS (that is, a par_marginal and a parents_frac0 file). 
   Sample includes parents observed at age 25-35, where age is defined by the higher earning parent, who have a child age 0. 
   Gets called by robustness.do, and uses the globals specified therein. 
   If running independently of robustness.do, specify paths in the globals commented out below. */
   
//global absmob_indir  "" //Path to cleaned data
//global absmob_outdir "" //Path to results folder

local cohort_min 1968
local cohort_max 1984

	set more off

	//Make the outdir if it doesn't exist
	ssc install confirmdir
	confirmdir "$absmob_outdir"
	if _rc > 0 { 
		mkdir "$absmob_outdir"
	}
	
	/* Reshape into 1 row per child */

	use "$absmob_indir/CPS_clean", clear
	keep if age <= 18
	rename id  id_child
	rename age age_child
	rename sex sex_child
	keep year id_child age_child sex_child mom_id pop_id
	
	tempfile cps_kids
	save    `cps_kids', replace
	
	use "$absmob_indir/CPS_clean", clear
	replace mom_id  = id 	 	if sex == 2
	replace mom_id  = id_spouse if sex_spouse == 2
	replace pop_id  = id 	 	if sex == 1
	replace pop_id  = id_spouse if sex_spouse == 1
	
	tempfile cps_parents
	save    `cps_parents', replace
	
	use `cps_kids', clear
	merge m:1 year mom_id pop_id using `cps_parents', keep(match) nogen
	
	// representative parent is set to richer
	foreach var in id age {
		gen rep_`var' = `var'
		replace rep_`var' = `var'_spouse if inctot == . & inctot_spouse != . 
		replace rep_`var' = `var'_spouse if inctot < inctot_spouse & inctot_spouse != .
	}
	keep if rep_age >= 25 & rep_age <= 35
	keep if age_child == 0
	
	// generate income variable
	tempvar incwelfr incwelfr_spouse 
	gen `incwelfr' = -incwelfr
	gen `incwelfr_spouse' = -incwelfr_spouse
	egen income = rowtotal(inctot inctot_spouse `incwelfr' `incwelfr_spouse')
	drop if mi(income) 

	//adjust for inflation	
	preserve	
	use "$absmob_indir/cpirs", clear
	sort year
	keep year rate
	tempfile inflation
	save    `inflation', replace
	keep if year == 2014
	local inflation_rate = rate[1]
	restore
	
	replace year = year - 1 //income refers to previous calendar year
	merge m:1 year using `inflation', assert(match using) keep(match master) nogen
	replace year = year + 1
	replace rate = rate / `inflation_rate'
	replace income = income/rate
	
	keep year income wtsupp
	
	save "$absmob_outdir/cps_par", replace

	foreach cohort of numlist `cohort_min'/`cohort_max' {
	
		use "$absmob_outdir/cps_par", clear
		
		keep if year == `cohort' 
			
		//drop zero (and neg) incomes
		su income if income > 0 [aw = wtsupp]
		local n_pos_income = r(N)
		su income [aw = wtsupp]
		local n_income = r(N)
		local frac0_`cohort' =  1 - (`n_pos_income' / `n_income')
				
		keep if income > 0
				
		fastxtile percentile = income [pw=wtsupp], n(100)
		collapse (mean) par_inc = income, by(percentile)
		keep percentile par_inc
		
		merge 1:1 percentile using "$absmob_indir/zero_incomes", nogen
		sort percentile
		//For all missing values, if _n = 2 or 1, set value to 0 else, set value to _N-1
		replace par_inc = cond(mi(par_inc), cond(_n<=2, 0, par_inc[_n-1]), par_inc)		
		replace par_inc = round(par_inc)			

		gen cohort = `cohort'
				
		tempfile par_`cohort'
		save    `par_`cohort'', replace 
	}
	use `par_`cohort_min'', clear
	local next = `cohort_min' + 1
	forval i = `next'/`cohort_max' {
		append using `par_`i''
	}
	order cohort percentile
	save "$absmob_outdir/par_marginal", replace

	keep cohort
	duplicates drop
	gen double frac0 = .
	foreach cohort of numlist `cohort_min'/`cohort_max' {
		replace frac0 = `frac0_`cohort'' if cohort == `cohort'
	}
	save "$absmob_outdir/parents_frac0", replace 



