/* Runs the code to produce the data sets needed for Fig 1-4 and appendix */


* Edit these globals to reflect your local paths 
global absmob_rawdir     "" //The path to folder containing downloaded data from IPUMS (see data_raw in README)
global absmob_indir      "" //The path to the cleaned data directory (see input_clean in README)
global absmob_resultsdir "" //The path to a local folder that will contain all the resulting datasets
global absmob_sourcesdir "" //The path to the folder containing the downloaded source code (that is, the .do- and .ado-files)

discard
adopath + "$absmob_sourcesdir"
clear
set more off

* Create Results Directory if it doesn't exist
ssc install confirmdir
confirmdir "$absmob_resultsdir"
if _rc > 0 { 
	mkdir "$absmob_resultsdir"
}

*-------------------------------------------------------------------------------
/* Baseline */
*-------------------------------------------------------------------------------

global absmob_outdir "$absmob_resultsdir/Baseline"
absmob compute all, cohorts(1940/1984) adj kid(immigrants(drop)) copula(copula_base_adjusted) debug(on)
use "$absmob_outdir/absmob", clear
merge m:1 cohort using "$absmob_outdir/parents_frac0", nogen // add in zero parent incomes
save "$absmob_outdir/figure_input_data", replace

// marginal distributions for matlab
use "$absmob_outdir/kid_marginal", clear
merge 1:1 percentile cohort using "$absmob_outdir/par_marginal", nogen assert(3)
order percentile kid_inc cohort par_inc
save "$absmob_outdir/marginals_baseline", replace
export delimited "$absmob_outdir/marginals_baseline.txt", replace novar

* Create Robustness Directory 
confirmdir "$absmob_resultsdir/Robustness"
if _rc > 0 { 
	mkdir "$absmob_resultsdir/Robustness"
}

*-------------------------------------------------------------------------------
/* 3a: deflators */
*-------------------------------------------------------------------------------

// PPI
global absmob_outdir "$absmob_resultsdir/Robustness/PPI"
absmob compute all, cohorts(1940/1984) adj income(inf(ppi)) kid(immigrants(drop)) debug(off)

// PCEPI
global absmob_outdir "$absmob_resultsdir/Robustness/PCEPI"
absmob compute all, cohorts(1940/1984) adj income(inf(pcepi)) kid(immigrants(drop)) debug(off) 

// CPI minus 0.8% growth
global absmob_outdir "$absmob_resultsdir/Robustness/CPI minus .8"
absmob compute all, cohorts(1940/1984) adj income(inflator(cpi08)) kid(immigrants(drop)) debug(off)


*-------------------------------------------------------------------------------
/* 3c: life cycle */
*-------------------------------------------------------------------------------

global absmob_outdir "$absmob_resultsdir/Robustness/40 vs 40"
absmob compute  all , cohorts(1940/1974) adj kid(age(40) immigrants(drop)) parent(age_at_census(35 45)) debug(off)


*-------------------------------------------------------------------------------
/* 3d: Adjusting for Family Size */
*-------------------------------------------------------------------------------

* Create Sons & Family Size Directories 
confirmdir "$absmob_resultsdir/Robustness/Sons/Fathers"
if _rc > 0 { 
	confirmdir "$absmob_resultsdir/Robustness/Sons"
	if _rc > 0 { 
		mkdir "$absmob_resultsdir/Robustness/Sons"
	}
	mkdir "$absmob_resultsdir/Robustness/Sons/Fathers"
}

confirmdir "$absmob_resultsdir/Robustness/Family Size"
if _rc > 0 { 
	mkdir "$absmob_resultsdir/Robustness/Family Size"
}

// Sons - Fathers: Individual Income
global absmob_outdir "$absmob_resultsdir/Robustness/Sons/Fathers/Individual Income"
absmob compute  all , cohorts(1940/1984) adj income(measure(indinc)) parent(gender(male)) kid(gender(male) immigrants(drop)) debug(off)

// Divide by Sqrt(Family Size)
global absmob_outdir "$absmob_resultsdir/Robustness/Family Size"
absmob compute  all , cohorts(1940/1984) adj income(divby(famsqrt)) kid(immigrants(drop)) debug(off)


*-------------------------------------------------------------------------------
/* 4: State-Level Analyses */
*-------------------------------------------------------------------------------

global absmob_outdir "$absmob_resultsdir/Robustness/States"
do "$absmob_sourcesdir/do_absmob_states.do"
/* Note that for the state-level analyses, the published results use the 1940 Full Population data and the 1980 5% sample for greater precision. 
   Though the default 1% samples produce very similar results, you should download those larger samples if you desire full reproducibility.*/

*-------------------------------------------------------------------------------
/* S2: Alternative CPI Growth */
*-------------------------------------------------------------------------------

// CPI minus 1% growth
global absmob_outdir "$absmob_resultsdir/Robustness/CPI minus 1"
absmob compute all, cohorts(1940/1984) adj kid(immigrants(drop)) income(inflator(cpi10)) debug(off)

// CPI minus 2% growth
global absmob_outdir "$absmob_resultsdir/Robustness/CPI minus 2"
absmob compute all, cohorts(1940/1984) adj kid(immigrants(drop)) income(inflator(cpi20)) debug(off)

// CPI-U
global absmob_outdir "$absmob_resultsdir/Robustness/CPI-U"
absmob compute all, cohorts(1940/1984) adj kid(immigrants(drop)) income(inflator(cpiu)) debug(off)

// GDP
global absmob_outdir "$absmob_resultsdir/Robustness/GDP"
absmob compute all, cohorts(1940/1984) adj income(inf(gdp)) kid(immigrants(drop)) debug(off) 


*-------------------------------------------------------------------------------
/* S3: family size */
*-------------------------------------------------------------------------------

// Divide by Nr of Adults
global absmob_outdir "$absmob_resultsdir/Robustness/Family Size/Nr of Adults"
absmob compute  all , cohorts(1940/1984) adj income(divby(spouses)) kid(immigrants(drop)) debug(off)

// Divide by Total Nr in Family
global absmob_outdir "$absmob_resultsdir/Robustness/Family Size/Total Nr in Family"
absmob compute  all , cohorts(1940/1984) adj income(divby(famsize)) kid(immigrants(drop)) debug(off)


*-------------------------------------------------------------------------------
/* S4: Raise by X */
*-------------------------------------------------------------------------------

* Create Raise by X Directory 
confirmdir "$absmob_resultsdir/Robustness/Raise by X"
if _rc > 0 { 
	mkdir "$absmob_resultsdir/Robustness/Raise by X"
}

foreach x of numlist 100 500 1000 5000(5000)50000 {

	confirmdir "$absmob_resultsdir/Robustness/Raise by X/`x'"
	if _rc > 0 { 
		mkdir "$absmob_resultsdir/Robustness/Raise by X/`x'"
	}
	use "$absmob_resultsdir/Baseline/par_marginal", clear
	save "$absmob_resultsdir/Robustness/Raise by X/`x'/par_marginal", replace
	use "$absmob_resultsdir/Baseline/parents_frac0", clear
	save "$absmob_resultsdir/Robustness/Raise by X/`x'/parents_frac0", replace
	
	use "$absmob_resultsdir/Baseline/kid_marginal", clear
	replace kid_inc = kid_inc + `x'
	save "$absmob_resultsdir/Robustness/Raise by X/`x'/kid_marginal", replace
	
	global absmob_outdir "$absmob_resultsdir/Robustness/Raise by X/`x'"
	absmob compute mobility, cohorts(1940/1984) adj kid(immigrants(drop)) debug(off)	
}


*-------------------------------------------------------------------------------
/* S5a Higher Thresholds */
*-------------------------------------------------------------------------------

confirmdir "$absmob_resultsdir/Robustness/Higher Thresholds"
if _rc > 0 { 
	mkdir "$absmob_resultsdir/Robustness/Higher Thresholds"
}

//requires that the baseline marginal distributions be up-to-date
use "$absmob_resultsdir/Baseline/kid_marginal", clear
merge 1:1 percentile cohort using "$absmob_resultsdir/Baseline/par_marginal", nogen
reshape wide kid_inc par_inc, i(percentile) j(cohort)

global absmob_outdir "$absmob_resultsdir/Robustness/Higher Thresholds"
save "$absmob_outdir/merged_cohorts", replace

copy "$absmob_resultsdir/Baseline/parents_frac0.dta" ///
	 "$absmob_outdir/parents_frac0.dta", replace
	 
	set more off
	matrix copula_mat = J(100,100,0)
	foreach c of numlist 1940/1984 {
		foreach threshold of numlist 80 90 100 110 120 {
		
			use "$absmob_resultsdir/Baseline/copula_base_adjusted_by_year", clear
			
			keep if cohort == `c'
	
			forval p = 1(1)100 {
				forval k = 1(1)100 {
					matrix copula_mat[`k',`p'] = p`p'[`k']
				}
			}
			
			use "$absmob_outdir/merged_cohorts", clear
			
			matrix abs_mob_`c'_`threshold' = J(100,100,0)
			matrix abs_mob_pr_`c'_`threshold' = J(100,100,0)
			forval p = 1(1)100 {
				forval k = 1(1)100 {
					local par_inc = par_inc`c'[`p']
					local kid_inc = kid_inc`c'[`k']
					if (`threshold'/100)*`par_inc' < `kid_inc' {
						matrix abs_mob_`c'_`threshold'[`k',`p'] = 1
						matrix abs_mob_pr_`c'_`threshold'[`k',`p'] = abs_mob_`c'_`threshold'[`k',`p'] * copula_mat[`k',`p']
					}
				}
			}
			preserve 
	
			* Save absolute mobility stats by year & percentile
			clear
			matrix T = abs_mob_pr_`c'_`threshold''
			svmat T
			egen abs_mob = rowtotal(_all)
			gen parent_percentile = _n		
			gen cohort = `c'
			gen threshold = `threshold'
	
			keep parent_percentile abs_mob cohort threshold
			tempfile abs_mob_`c'_`threshold'
			save `abs_mob_`c'_`threshold''
	
			restore
			mata : st_matrix("abs`c'", colsum(rowsum(st_matrix("abs_mob_pr_`c'"))))
			}
		use `abs_mob_`c'_80', clear
		foreach t of numlist 90 100 110 120 {
			append using `abs_mob_`c'_`t''
		}
		tempfile abs_mob_`c'
		save `abs_mob_`c''
	}
	
	//combine all cohort-wise abs-mobs into a single file
	use `abs_mob_1940', clear
	foreach cohort of numlist 1941/1984 {
		append using `abs_mob_`cohort'' 
	}				
		
	//set mobility = 1 for all parent-percentiles with <= 0 income
	merge m:1 cohort using "$absmob_outdir/parents_frac0", nogen
	sort cohort threshold parent_percentile
	gen abs_mob_ad = 1 if parent_percentile <= 100 * frac0
	replace abs_mob_ad = (1-frac0)*abs_mob if parent_percentile > 100 * frac0
	rename abs_mob abs_mob_pos_par
	rename abs_mob_ad abs_mob			

	keep cohort threshold parent_percentile abs_mob* 
	save "$absmob_outdir/absmob", replace

	
*-------------------------------------------------------------------------------
/* S6: Other Income Definitions */
*-------------------------------------------------------------------------------

// Wages
global absmob_outdir "$absmob_resultsdir/Robustness/Wages"
absmob compute all, cohorts(1940/1984) adj income(measure(incwage))

// Total Family Income
global absmob_outdir "$absmob_resultsdir/Robustness/Family Income"
absmob compute all, cohorts(1940/1984) adj income(measure(faminc))


*-------------------------------------------------------------------------------
/* S7: Immigrants */
*-------------------------------------------------------------------------------

// Include immigrants 
global absmob_outdir "$absmob_resultsdir/Robustness/Immigrants"
absmob compute all, cohorts(1940/1984) adj kid(immigrants(keep)) debug(off)


*-------------------------------------------------------------------------------
/* S8: Parent Matching Age */
*-------------------------------------------------------------------------------

// Parent and Income Defined ages 25-35
global absmob_outdir "$absmob_resultsdir/Robustness/Parent 25-35"
absmob compute all, cohorts(1940/1984) adj parent(age_at_census(25 35) age_at_childbirth(25 35)) debug(off)


*-------------------------------------------------------------------------------
/* S9: CPS-CPS & Census-Census */
*-------------------------------------------------------------------------------

// S9a: CPS-CPS. Requires that the baseline marginal distributions be up-to-date  

global absmob_outdir "$absmob_resultsdir/Robustness/CPS-CPS"

do "$absmob_sourcesdir/par_marginal_cps.do"

use "$absmob_resultsdir/Baseline/kid_marginal", clear
keep if cohort > 1967
save "$absmob_resultsdir/Robustness/CPS-CPS/kid_marginal", replace

absmob compute mobility, cohorts(1968/1984) adj debug(off)

// S9b: Census-Census. Requires that the baseline marginal distributions be up-to-date

global absmob_outdir "$absmob_resultsdir/Robustness/Census-Census"

do "$absmob_sourcesdir/kid_marginal_census.do"

use "$absmob_resultsdir/Baseline/par_marginal", clear
keep if inlist(cohort, 1940, 1950, 1960, 1970, 1980)
save "$absmob_resultsdir/Robustness/Census-Census/par_marginal", replace
copy "$absmob_resultsdir/Baseline/parents_frac0.dta" ///
	 "$absmob_resultsdir/Robustness/Census-Census/parents_frac0.dta"

use "$absmob_resultsdir/Baseline/copula_base_adjusted_by_year", clear
keep if inlist(cohort, 1940, 1950, 1960, 1970, 1980)
save "$absmob_resultsdir/Robustness/Census-Census/copula_base_adjusted_by_year", replace

absmob compute mobility, cohorts(1940(10)1980) adj debug(off)

*-------------------------------------------------------------------------------
/* S10: Gender */
*-------------------------------------------------------------------------------

* Create Sons & Family Size Directories 
confirmdir "$absmob_resultsdir/Robustness/Daughters"
if _rc > 0 { 
	mkdir "$absmob_resultsdir/Robustness/Daughters"
}

// Daughters - Fathers: Individual Income
global absmob_outdir "$absmob_resultsdir/Robustness/Daughters/individual"
absmob compute  all , cohorts(1940/1984) adj income(measure(indinc)) parent(gender(male)) kid(gender(female) immigrants(drop)) debug(off)

// Daughters - Fathers: Household Income
global absmob_outdir "$absmob_resultsdir/Robustness/Daughters/household"
absmob compute all , cohorts(1940/1984) adj kid(gender(female) immigrants(drop)) debug(off)

// Sons - Fathers: Household Income
global absmob_outdir "$absmob_resultsdir/Robustness/Sons/Fathers/household income"
absmob compute all , cohorts(1940/1984) adj kid(gender(male) immigrants(drop)) debug(off)


*-------------------------------------------------------------------------------
/* S11: Median Income Trends */
*-------------------------------------------------------------------------------

confirmdir "$absmob_resultsdir/Robustness/Median Income Trends"
if _rc > 0 { 
	mkdir "$absmob_resultsdir/Robustness/Median Income Trends"
}

global absmob_outdir "$absmob_resultsdir/Robustness/Median Income Trends"
do "$absmob_sourcesdir/CPS_marginal_check.do"


*-------------------------------------------------------------------------------
/* Online Table: Min Cutoffs for Marginals */
*-------------------------------------------------------------------------------

global absmob_outdir "$absmob_resultsdir/Robustness/Min Cutoffs"

absmob_min compute par, cohorts(1940/1984) adj kid(immigrants(drop)) debug(off)
absmob_min compute kid, cohorts(1940/1984) adj kid(immigrants(drop)) debug(off)
